@@ -37,6 +37,7 @@ typedef enum
   GABBLE_BYTESTREAM_STATE_INITIATING,
   /* Bytestream open */
   GABBLE_BYTESTREAM_STATE_OPEN,
+  GABBLE_BYTESTREAM_STATE_CLOSING,
   GABBLE_BYTESTREAM_STATE_CLOSED,
   NUM_GABBLE_BYTESTREAM_STATES,
 } GabbleBytestreamState;
@@ -56,6 +57,7 @@ struct _GabbleBytestreamIfaceClass {
   void (*close) (GabbleBytestreamIface *bytestream, GError *error);
   void (*accept) (GabbleBytestreamIface *bytestream,
       GabbleBytestreamAugmentSiAcceptReply func, gpointer user_data);
+  void (*block_reading) (GabbleBytestreamIface *bytestream, gboolean block);
 };
 
 GType gabble_bytestream_iface_get_type (void);
@@ -83,6 +85,9 @@ void gabble_bytestream_iface_close (GabbleBytestreamIface *bytestream,
 void gabble_bytestream_iface_accept (GabbleBytestreamIface *bytestream,
     GabbleBytestreamAugmentSiAcceptReply func, gpointer user_data);
 
+void gabble_bytestream_iface_block_reading (GabbleBytestreamIface *bytestream,
+    gboolean block);
+
 G_END_DECLS
 
 #endif /* #ifndef __GABBLE_BYTESTREAM_IFACE_H__ */
