@@ -494,6 +494,13 @@ gabble_file_transfer_channel_constructor (GType type,
        tp_handle_inspect (contact_repo, self->priv->initiator),
        self->priv->filename, self->priv->size);
 
+  if (self->priv->initiator == base_conn->self_handle)
+    {
+      /* Outgoing FT , we'll need SOCK5 proxies when we'll offer the file */
+      gabble_bytestream_factory_query_socks5_proxies (
+          self->priv->connection->bytestream_factory);
+    }
+
   return obj;
 }
 
@@ -1342,9 +1349,6 @@ data_received_cb (GabbleBytestreamIface *stream,
       return;
     }
 
-  DEBUG ("received %"G_GSIZE_FORMAT" bytes from bytestream. Writing to socket",
-      data->len);
-
   transferred_chunk (self, (guint64) data->len);
 
   if (self->priv->transferred_bytes + self->priv->initial_offset >=
@@ -1365,7 +1369,6 @@ data_received_cb (GabbleBytestreamIface *stream,
   if (!gibber_transport_buffer_is_empty (self->priv->transport))
     {
       /* We don't want to send more data while the buffer isn't empty */
-      DEBUG ("file transfer buffer isn't empty. Block the bytestream");
       gabble_bytestream_iface_block_reading (self->priv->bytestream, TRUE);
     }
 }
@@ -1604,9 +1607,6 @@ transport_handler (GibberTransport *transport,
 {
   GabbleFileTransferChannel *self = GABBLE_FILE_TRANSFER_CHANNEL (user_data);
 
-  DEBUG ("Data available, writing a %"G_GSIZE_FORMAT" bytes chunk",
-      data->length);
-
   if (!gabble_bytestream_iface_send (self->priv->bytestream, data->length,
         (const gchar *) data->data))
     {
@@ -1637,17 +1637,7 @@ bytestream_write_blocked_cb (GabbleBytestreamIface *bytestream,
                              gboolean blocked,
                              GabbleFileTransferChannel *self)
 {
-  if (blocked)
-    {
-      DEBUG ("bytestream blocked, stop to read data from FT socket");
-    }
-  else
-    {
-      DEBUG ("bytestream unblocked, restart to read data from FT socket");
-    }
-
   gibber_transport_block_receiving (self->priv->transport, blocked);
-
 }
 
 static void
@@ -1688,7 +1678,6 @@ transport_buffer_empty_cb (GibberTransport *transport,
                            GabbleFileTransferChannel *self)
 {
   /* Buffer is empty so we can unblock the buffer if it was blocked */
-  DEBUG ("file transfer buffer is empty. Unblock the bytestream");
   gabble_bytestream_iface_block_reading (self->priv->bytestream, FALSE);
 
   if (self->priv->state > TP_FILE_TRANSFER_STATE_OPEN)
