@@ -66,10 +66,17 @@ GType gabble_im_channel_get_type (void);
                               GabbleIMChannelClass))
 
 void _gabble_im_channel_receive (GabbleIMChannel *chan,
-    TpChannelTextMessageType type, TpHandle sender, const char *from,
-    time_t timestamp, const char *id, const char *text,
-    TpChannelTextSendError send_error, TpDeliveryStatus delivery_status);
-void _gabble_im_channel_state_receive (GabbleIMChannel *chan, guint state);
+    TpChannelTextMessageType type,
+    TpHandle sender,
+    const char *from,
+    time_t timestamp,
+    const char *id,
+    const char *text,
+    TpChannelTextSendError send_error,
+    TpDeliveryStatus delivery_status,
+    gint state);
+void _gabble_im_channel_state_receive (GabbleIMChannel *chan,
+    TpChannelChatState state);
 
 G_END_DECLS
 
