@@ -65,7 +65,8 @@ typedef enum {
   JINGLE_ACTION_SESSION_INITIATE,
   JINGLE_ACTION_SESSION_TERMINATE,
   JINGLE_ACTION_TRANSPORT_INFO,
-  JINGLE_ACTION_TRANSPORT_ACCEPT
+  JINGLE_ACTION_TRANSPORT_ACCEPT,
+  JINGLE_ACTION_DESCRIPTION_INFO
 } JingleAction;
 
 typedef enum {
@@ -79,7 +80,7 @@ typedef enum {
   JINGLE_TRANSPORT_UNKNOWN,
   JINGLE_TRANSPORT_GOOGLE_P2P,
   JINGLE_TRANSPORT_RAW_UDP,
-  JINGLE_TRANSPORT_ICE
+  JINGLE_TRANSPORT_ICE_UDP,
 } JingleTransportType;
 
 typedef enum {
@@ -139,8 +140,17 @@ GType gabble_jingle_factory_lookup_transport (GabbleJingleFactory *self,
 void _jingle_factory_unregister_session (GabbleJingleFactory *factory,
     const gchar *sid);
 
-GabbleJingleSession *gabble_jingle_factory_create_session (GabbleJingleFactory
-    *fac, TpHandle peer, const gchar *peer_resource);
+GabbleJingleSession *gabble_jingle_factory_create_session (
+    GabbleJingleFactory *fac,
+    TpHandle peer,
+    const gchar *peer_resource,
+    gboolean local_hold);
+
+typedef void (*GabbleJingleFactoryRelaySessionCb) (GPtrArray *relays,
+    gpointer user_data);
+void gabble_jingle_factory_create_google_relay_session (
+    GabbleJingleFactory *self, guint components,
+    GabbleJingleFactoryRelaySessionCb callback, gpointer user_data);
 
 const gchar *gabble_jingle_factory_get_google_relay_token (
     GabbleJingleFactory *self);
@@ -148,6 +158,8 @@ const gchar *gabble_jingle_factory_get_google_relay_token (
 gboolean gabble_jingle_factory_get_stun_server (GabbleJingleFactory *self,
     gchar **stun_server, guint *stun_port);
 
+void gabble_jingle_factory_set_test_mode (void);
+
 G_END_DECLS;
 
 #endif /* __JINGLE_FACTORY_H__ */
