@@ -54,7 +54,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleOlpcActivityViewPrivate GabbleOlpcActivityViewPrivate;
 struct _GabbleOlpcActivityViewPrivate
 {
   GHashTable *properties;
@@ -69,10 +68,6 @@ G_DEFINE_TYPE_WITH_CODE (
       NULL);
     );
 
-#define GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE(obj) \
-    ((GabbleOlpcActivityViewPrivate *) obj->priv)
-
-
 static void
 gabble_olpc_activity_view_init (GabbleOlpcActivityView *self)
 {
@@ -88,8 +83,7 @@ static void
 gabble_olpc_activity_view_dispose (GObject *object)
 {
   GabbleOlpcActivityView *self = GABBLE_OLPC_ACTIVITY_VIEW (object);
-  GabbleOlpcActivityViewPrivate *priv = GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (
-      self);
+  GabbleOlpcActivityViewPrivate *priv = self->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -104,8 +98,7 @@ static void
 gabble_olpc_activity_view_finalize (GObject *object)
 {
   GabbleOlpcActivityView *self = GABBLE_OLPC_ACTIVITY_VIEW (object);
-  GabbleOlpcActivityViewPrivate *priv = GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (
-      self);
+  GabbleOlpcActivityViewPrivate *priv = self->priv;
 
   g_hash_table_destroy (priv->properties);
   tp_handle_set_destroy (priv->participants);
@@ -121,8 +114,7 @@ gabble_olpc_activity_view_get_property (GObject *object,
                                         GParamSpec *pspec)
 {
   GabbleOlpcActivityView *self = GABBLE_OLPC_ACTIVITY_VIEW (object);
-  GabbleOlpcActivityViewPrivate *priv = GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (
-      self);
+  GabbleOlpcActivityViewPrivate *priv = self->priv;
 
   switch (property_id)
     {
@@ -166,8 +158,7 @@ gabble_olpc_activity_view_set_property (GObject *object,
                                         GParamSpec *pspec)
 {
   GabbleOlpcActivityView *self = GABBLE_OLPC_ACTIVITY_VIEW (object);
-  GabbleOlpcActivityViewPrivate *priv = GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (
-      self);
+  GabbleOlpcActivityViewPrivate *priv = self->priv;
 
   switch (property_id)
     {
@@ -221,23 +212,18 @@ gabble_olpc_activity_view_constructor (GType type,
   GabbleOlpcActivityViewPrivate *priv;
   DBusGConnection *bus;
   TpBaseConnection *conn;
-  TpHandleRepoIface *contact_handles;
 
   obj = G_OBJECT_CLASS (gabble_olpc_activity_view_parent_class)->
            constructor (type, n_props, props);
 
   view = GABBLE_OLPC_VIEW (obj);
 
-  priv = GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (GABBLE_OLPC_ACTIVITY_VIEW (
-        obj));
+  priv = GABBLE_OLPC_ACTIVITY_VIEW (obj)->priv;
   conn = (TpBaseConnection *) view->conn;
 
   bus = tp_get_bus ();
   dbus_g_connection_register_g_object (bus, view->object_path, obj);
 
-  contact_handles = tp_base_connection_get_handles (conn,
-      TP_HANDLE_TYPE_CONTACT);
-
   if (priv->properties == NULL)
     {
       priv->properties = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -299,8 +285,7 @@ gabble_olpc_activity_view_send_request (GabbleOlpcView *view,
                                         GError **error)
 {
   GabbleOlpcActivityView *self = GABBLE_OLPC_ACTIVITY_VIEW (view);
-  GabbleOlpcActivityViewPrivate *priv = \
-      GABBLE_OLPC_ACTIVITY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcActivityViewPrivate *priv = self->priv;
   LmMessage *query;
   LmMessageNode *activity_node;
   gchar *max_str, *id_str;
