@@ -661,10 +661,10 @@ gabble_private_tubes_factory_free_feat (gpointer data)
 static gpointer
 gabble_private_tubes_factory_parse_caps (
     GabbleCapsChannelManager *manager,
-    LmMessageNode *query_result)
+    gchar **uris)
 {
   TubesCapabilities *caps;
-  NodeIter i;
+  gchar **i;
 
   caps = g_new0 (TubesCapabilities, 1);
   caps->stream_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -672,41 +672,32 @@ gabble_private_tubes_factory_parse_caps (
   caps->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
       g_free, gabble_private_tubes_factory_free_feat);
 
-  for (i = node_iter (query_result); i; i = node_iter_next (i))
+  for (i = uris; *i; i++)
     {
-      LmMessageNode *child = node_iter_data (i);
-      const gchar *var;
+      const gchar *uri = *i;
 
-      if (0 != strcmp (child->name, "feature"))
-        continue;
-
-      var = lm_message_node_get_attribute (child, "var");
-
-      if (NULL == var)
-        continue;
-
-      if (!g_str_has_prefix (var, NS_TUBES))
+      if (!g_str_has_prefix (uri, NS_TUBES))
         continue;
 
       /* tubes generic cap or service specific */
       caps->tubes_supported = TRUE;
 
-      if (g_str_has_prefix (var, NS_TUBES "/"))
+      if (g_str_has_prefix (uri, NS_TUBES "/"))
         {
           /* http://telepathy.freedesktop.org/xmpp/tubes/$type#$service */
-          var += strlen (NS_TUBES "/");
-          if (g_str_has_prefix (var, "stream#"))
+          uri += strlen (NS_TUBES "/");
+          if (g_str_has_prefix (uri, "stream#"))
             {
               gchar *service;
-              var += strlen ("stream#");
-              service = g_strdup (var);
+              uri += strlen ("stream#");
+              service = g_strdup (uri);
               g_hash_table_insert (caps->stream_tube_caps, service, NULL);
             }
-          else if (g_str_has_prefix (var, "dbus#"))
+          else if (g_str_has_prefix (uri, "dbus#"))
             {
               gchar *service;
-              var += strlen ("dbus#");
-              service = g_strdup (var);
+              uri += strlen ("dbus#");
+              service = g_strdup (uri);
               g_hash_table_insert (caps->dbus_tube_caps, service, NULL);
             }
         }
@@ -726,23 +717,19 @@ gabble_private_tubes_factory_free_caps (
  g_free (caps);
 }
 
-static void
-copy_caps_helper (gpointer key, gpointer value, gpointer user_data)
+static Feature *
+copy_caps (Feature *feat)
 {
-  GHashTable *out = user_data;
-  gchar *str = key;
-  Feature *feat = value;
-  Feature *copy = NULL;
+  Feature *copy;
 
-  if (value != NULL)
-    {
-      copy = g_new0 (Feature, 1);
-      copy->feature_type = feat->feature_type;
-      copy->ns = g_strdup (feat->ns);
-      copy->caps = feat->caps;
-    }
+  if (feat == NULL)
+    return NULL;
 
-  g_hash_table_insert (out, g_strdup (str), copy);
+  copy = g_new0 (Feature, 1);
+  copy->feature_type = feat->feature_type;
+  copy->ns = g_strdup (feat->ns);
+  copy->caps = feat->caps;
+  return copy;
 }
 
 static void
@@ -756,13 +743,15 @@ gabble_private_tubes_factory_copy_caps (
 
   caps_out->stream_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
       g_free, gabble_private_tubes_factory_free_feat);
-  g_hash_table_foreach (caps_in->stream_tube_caps, copy_caps_helper,
-      caps_out->stream_tube_caps);
+  tp_g_hash_table_update (caps_out->stream_tube_caps,
+      caps_in->stream_tube_caps, (GBoxedCopyFunc) g_strdup,
+      (GBoxedCopyFunc) copy_caps);
 
   caps_out->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
       g_free, gabble_private_tubes_factory_free_feat);
-  g_hash_table_foreach (caps_in->dbus_tube_caps, copy_caps_helper,
-      caps_out->dbus_tube_caps);
+  tp_g_hash_table_update (caps_out->dbus_tube_caps,
+      caps_in->dbus_tube_caps, (GBoxedCopyFunc) g_strdup,
+      (GBoxedCopyFunc) copy_caps);
 
   caps_out->tubes_supported = caps_in->tubes_supported;
 
@@ -782,9 +771,11 @@ gabble_private_tubes_factory_update_caps (
     return;
 
   tp_g_hash_table_update (caps_out->stream_tube_caps,
-      caps_in->stream_tube_caps, (GBoxedCopyFunc) g_strdup, NULL);
+      caps_in->stream_tube_caps, (GBoxedCopyFunc) g_strdup,
+      (GBoxedCopyFunc) copy_caps);
   tp_g_hash_table_update (caps_out->dbus_tube_caps,
-      caps_in->dbus_tube_caps, (GBoxedCopyFunc) g_strdup, NULL);
+      caps_in->dbus_tube_caps, (GBoxedCopyFunc) g_strdup,
+      (GBoxedCopyFunc) copy_caps);
 }
 
 static gboolean
@@ -845,12 +836,11 @@ gabble_private_tubes_factory_caps_diff (
 }
 
 static void
-gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
-                                      GabbleConnection *conn,
-                                      TpHandle handle,
-                                      GHashTable *cap)
+gabble_private_tubes_factory_add_self_capability (
+    GabbleCapsChannelManager *manager,
+    GabbleConnection *conn,
+    GHashTable *cap)
 {
-  TpBaseConnection *base = (TpBaseConnection *) conn;
   GabblePresence *presence;
   TubesCapabilities *caps;
   const gchar *channel_type;
@@ -868,11 +858,7 @@ gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
         TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_CONTACT)
     return;
 
-  if (handle == base->self_handle)
-    presence = conn->self_presence;
-  else
-    presence = gabble_presence_cache_get (conn->presence_cache, handle);
-
+  presence = conn->self_presence;
   g_assert (presence != NULL);
 
   if (presence->per_channel_manager_caps == NULL)
@@ -888,18 +874,17 @@ gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
           g_free, gabble_private_tubes_factory_free_feat);
       g_hash_table_insert (presence->per_channel_manager_caps, manager, caps);
 
-      if (handle == base->self_handle)
-        /* We always support generic tubes caps */
-        caps->tubes_supported = TRUE;
+      /* We always support generic tubes caps */
+      caps->tubes_supported = TRUE;
     }
 
   if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
-      Feature *feat = g_new0 (Feature, 1);
       gchar *service = g_strdup (tp_asv_get_string (cap,
           TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service"));
       if (service != NULL)
         {
+          Feature *feat = g_new0 (Feature, 1);
           feat->feature_type = FEATURE_OPTIONAL;
           feat->ns = g_strdup_printf ("%s/stream#%s", NS_TUBES, service);
           feat->caps = 0;
@@ -908,11 +893,11 @@ gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
     }
   else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
-      Feature *feat = g_new0 (Feature, 1);
       gchar *service = g_strdup (tp_asv_get_string (cap,
           TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName"));
       if (service != NULL)
         {
+          Feature *feat = g_new0 (Feature, 1);
           feat->feature_type = FEATURE_OPTIONAL;
           feat->ns = g_strdup_printf ("%s/dbus#%s", NS_TUBES, service);
           feat->caps = 0;
@@ -1399,5 +1384,5 @@ caps_channel_manager_iface_init (gpointer g_iface,
   iface->copy_caps = gabble_private_tubes_factory_copy_caps;
   iface->update_caps = gabble_private_tubes_factory_update_caps;
   iface->caps_diff = gabble_private_tubes_factory_caps_diff;
-  iface->add_cap = gabble_private_tubes_factory_add_cap;
+  iface->add_self_capability = gabble_private_tubes_factory_add_self_capability;
 }
