@@ -1516,6 +1516,10 @@ gabble_tube_stream_constructor (GType type,
   else
     {
       priv->state = TP_TUBE_CHANNEL_STATE_LOCAL_PENDING;
+
+      /* We'll need SOCKS5 proxies if the tube is accepted */
+      gabble_bytestream_factory_query_socks5_proxies (
+          priv->conn->bytestream_factory);
     }
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
@@ -2354,8 +2358,15 @@ gabble_tube_stream_get_supported_socket_types (void)
       1);
   access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
   g_array_append_val (unix_tab, access_control);
-  access_control = TP_SOCKET_ACCESS_CONTROL_CREDENTIALS;
-  g_array_append_val (unix_tab, access_control);
+
+  /* Credentials-passing is non-portable, so only advertise it on platforms
+   * where we have an implementation (like Linux) */
+  if (gibber_unix_transport_supports_credentials ())
+    {
+      access_control = TP_SOCKET_ACCESS_CONTROL_CREDENTIALS;
+      g_array_append_val (unix_tab, access_control);
+    }
+
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_UNIX),
       unix_tab);
 
