@@ -1000,6 +1000,34 @@ vcard_copy (LmMessageNode *parent, LmMessageNode *src)
     return new;
 }
 
+static gboolean
+vcard_node_changed (GabbleConnection *conn,
+                    const gchar *key,
+                    const gchar *value,
+                    LmMessageNode *vcard_node)
+{
+  LmMessageNode *node;
+
+  if (conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER &&
+      strcmp (key, "N") != 0 && strcmp (key, "FN") != 0 &&
+      strcmp (key, "PHOTO") != 0)
+    {
+      return FALSE;
+    }
+
+  node = lm_message_node_get_child (vcard_node, key);
+  if (node != NULL)
+    {
+      const gchar *node_value = lm_message_node_get_value (node);
+
+      if (!tp_strdiff (node_value, value))
+        return FALSE;
+    }
+
+  DEBUG ("vcard node %s changed, vcard needs update", key);
+  return TRUE;
+}
+
 static void
 manager_patch_vcard (GabbleVCardManager *self,
                      LmMessageNode *vcard_node)
@@ -1008,6 +1036,9 @@ manager_patch_vcard (GabbleVCardManager *self,
   LmMessage *msg;
   LmMessageNode *patched_vcard;
   GList *li;
+  GHashTableIter iter;
+  gpointer key, value;
+  gboolean vcard_changed = FALSE;
 
   /* Bail out if we don't have outstanding edits to make, or if we already
    * have a set request in progress.
@@ -1015,6 +1046,22 @@ manager_patch_vcard (GabbleVCardManager *self,
   if (priv->edits == NULL || priv->edit_pipeline_item != NULL)
       return;
 
+  g_hash_table_iter_init (&iter, priv->edits);
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      if (vcard_node_changed (priv->connection, key, value, vcard_node))
+        {
+          vcard_changed = TRUE;
+          break;
+        }
+    }
+
+  if (!vcard_changed)
+    {
+      DEBUG ("nothing changed, not updating vcard");
+      goto out;
+    }
+
   DEBUG("patching vcard");
 
   msg = lm_message_new_with_sub_type (NULL, LM_MESSAGE_TYPE_IQ,
@@ -1036,6 +1083,7 @@ manager_patch_vcard (GabbleVCardManager *self,
 
   lm_message_unref (msg);
 
+out:
   /* We've applied those, forget about them */
   g_hash_table_destroy (priv->edits);
   priv->edits = NULL;
@@ -1061,6 +1109,13 @@ suspended_request_timeout_cb (gpointer data)
   return FALSE;
 }
 
+static gboolean
+is_item_not_found (const GError *error)
+{
+  return (error->domain == GABBLE_XMPP_ERROR &&
+      error->code == XMPP_ERROR_ITEM_NOT_FOUND);
+}
+
 /* Called when a GET request in the pipeline has either succeeded or failed. */
 static void
 pipeline_reply_cb (GabbleConnection *conn,
@@ -1086,7 +1141,11 @@ pipeline_reply_cb (GabbleConnection *conn,
 
   entry->pipeline_item = NULL;
 
-  if (error)
+  /* XEP-0054 says that the server MUST return <item-not-found/> if you have no
+   * vCard set, so we should treat that case identically to the server
+   * returning success but with no <vCard/> node.
+   */
+  if (error != NULL && !is_item_not_found (error))
     {
       /* First, handle the error "wait": suspend the request and replay it
        * later */
