@@ -8,6 +8,7 @@ import os
 import hashlib
 import sys
 import random
+import re
 import traceback
 
 import ns
@@ -61,12 +62,6 @@ def send_error_reply(stream, iq, error_stanza=None):
 
 def request_muc_handle(q, conn, stream, muc_jid):
     servicetest.call_async(q, conn, 'RequestHandles', 2, [muc_jid])
-    host = muc_jid.split('@')[1]
-    event = q.expect('stream-iq', to=host, query_ns=ns.DISCO_INFO)
-    result = make_result_iq(stream, event.stanza)
-    feature = result.firstChildElement().addElement('feature')
-    feature['var'] = ns.MUC
-    stream.send(result)
     event = q.expect('dbus-return', method='RequestHandles')
     return event.value[0][0]
 
@@ -306,8 +301,11 @@ class GoogleXmlStream(BaseXmlStream):
             self.send(iq)
 
 def make_connection(bus, event_func, params=None):
+    # Gabble accepts a resource in 'account', but the value of 'resource'
+    # overrides it if there is one.
+    account = 'test@localhost/%s' % re.sub(r'.*tests/twisted/', '', sys.argv[0])
     default_params = {
-        'account': 'test@localhost/Resource',
+        'account': account,
         'password': 'pass',
         'resource': 'Resource',
         'server': 'localhost',
