@@ -87,9 +87,40 @@ class JingleProtocol:
         "Creates a <transport> element for Google P2P transport"
         return ('transport', ns.GOOGLE_P2P, {}, [])
 
-    def TransportIceUdp(self):
-        "Creates a <transport> element for ICE-UDP transport"
-        return ('transport', ns.JINGLE_TRANSPORT_ICEUDP, {}, [])
+    def TransportIceUdp(self, remote_transports=[]):
+        """
+        Creates a <transport> element for ICE-UDP transport.
+        If remote_transports is present, and of the form
+        [(host, port, proto, subtype, profile, pref, transtype, user, pwd)]
+        (basically a list of Media_Stream_Handler_Transport without the
+        component number) then it will be converted to xml and added.
+        """
+        candidates = []
+        attrs = {}
+        for (host, port, proto, subtype, profile, pref, transtype, user, pwd
+                ) in remote_transports:
+            if "ufrag" not in attrs:
+                attrs = {"ufrag": user, "pwd": pwd}
+            else:
+                assert (user == attrs["ufrag"] and pwd == attrs["pwd"]
+                    ), "user and pwd should be the same across all candidates."
+
+            node = ("candidate", None, {
+                # ICE-CORE says it can be arbitrary string, even though XEP
+                # gives an int as an example.
+                "foundation": "fake",
+                "ip": host,
+                "port": str(port),
+                "protocol": ["udp", "tcp"][proto],
+                # Gabble multiplies by 65536 so we should too.
+                "priority": str(int(pref * 65536)),
+                "type":  ["host", "srflx", "srflx"][transtype],
+                "network": "0",
+                "generation": "0",# Increment this yourself if you care.
+                "component": "1", # 1 is rtp, 2 is rtcp
+                }, []) #NOTE: subtype and profile are unused
+            candidates.append(node)
+        return ('transport', ns.JINGLE_TRANSPORT_ICEUDP, attrs, candidates)
 
     def Presence(self, frm, to, caps):
         "Creates <presence> stanza with specified capabilities"
@@ -119,11 +150,11 @@ class JingleProtocol:
     def match_jingle_action(self, q, action):
         return q is not None and q.name == 'jingle' and q['action'] == action
 
-    def extract_session_id(self, query):
+    def _extract_session_id(self, query):
         return query['sid']
 
     def validate_session_initiate(self, query):
-        return True
+        raise NotImplementedError()
 
     def can_do_video(self):
         return True
@@ -207,14 +238,15 @@ class GtalkProtocol03(JingleProtocol):
     def TransportGoogleP2P(self):
         return None
 
-    def extract_session_id(self, query):
+    def _extract_session_id(self, query):
         return query['id']
 
     def can_do_video_only(self):
         return False
 
     def validate_session_initiate(self, query):
-        assert query['id'] != None
+        sid = self._extract_session_id(query)
+
         # No transport in GTalk03
         assert xpath.queryForNodes('/session/transport', query) == None
 
@@ -232,8 +264,9 @@ class GtalkProtocol03(JingleProtocol):
             assert xpath.queryForNodes(
                 '/session/description/payload-type[@xmlns="%s"]' %
                     ns.GOOGLE_SESSION_PHONE, query)
-
-        return True
+            return (sid, ['fake-audio'], ['fake-video'])
+        else:
+            return (sid, ['fake-audio'], [])
 
     def seperate_contents(self):
         return False
@@ -279,9 +312,13 @@ class GtalkProtocol04(JingleProtocol):
         action = self._action_map(action)
         return q is not None and q.name == 'session' and q['type'] == action
 
-    def extract_session_id(self, query):
+    def _extract_session_id(self, query):
         return query['id']
 
+    def validate_session_initiate(self, query):
+        # FIXME: validate it!
+        return (self._extract_session_id(query), ['fake-audio'], [])
+
     def can_do_video(self):
         return False
 
@@ -312,6 +349,36 @@ class JingleProtocol015(JingleProtocol):
             namespace = 'unexistent-namespace'
         return ('description', namespace, { 'type': type }, children)
 
+    def validate_session_initiate(self, query):
+        contents = xpath.queryForNodes(
+            '/jingle[@xmlns="%s"]/content' % ns.JINGLE_015,
+            query)
+
+        audio, video = [], []
+
+        for c in contents:
+            a_desc = xpath.queryForNodes(
+                '/content/description[@xmlns="%s"]' % ns.JINGLE_015_AUDIO,
+                c)
+            v_desc = xpath.queryForNodes(
+                '/content/description[@xmlns="%s"]' % ns.JINGLE_015_VIDEO,
+                c)
+
+            if a_desc is not None:
+                assert len(a_desc) == 1, c.toXml()
+                assert v_desc is None
+                audio.append(c['name'])
+            elif v_desc is not None:
+                assert len(v_desc) == 1, c.toXml()
+                assert a_desc is None
+                video.append(c['name'])
+            else:
+                assert False, c.toXml()
+
+        assert len(audio) + len(video) > 0, query.toXml()
+
+        return (self._extract_session_id(query), audio, video)
+
 class JingleProtocol031(JingleProtocol):
     features = [ ns.JINGLE, ns.JINGLE_RTP, ns.JINGLE_RTP_AUDIO,
         ns.JINGLE_RTP_VIDEO, ns.GOOGLE_P2P ]
@@ -346,6 +413,33 @@ class JingleProtocol031(JingleProtocol):
 
         return EventPattern('stream-iq', predicate=p)
 
+    def validate_session_initiate(self, query):
+        contents = xpath.queryForNodes(
+            '/jingle[@xmlns="%s"]/content' % ns.JINGLE,
+            query)
+
+        audio, video = [], []
+
+        for c in contents:
+            descs = xpath.queryForNodes(
+                '/content/description[@xmlns="%s"]' % ns.JINGLE_RTP,
+                c)
+
+            assert len(descs) == 1, c.toXml()
+
+            d = descs[0]
+
+            if d['media'] == 'audio':
+                audio.append(c['name'])
+            elif d['media'] == 'video':
+                video.append(c['name'])
+            else:
+                assert False, c.toXml()
+
+        assert len(audio) + len(video) > 0, query.toXml()
+
+        return (self._extract_session_id(query), audio, video)
+
 class JingleTest2:
     # Default caps for the remote end
     remote_caps = { 'ext': '', 'ver': '0.0.0',
@@ -423,35 +517,19 @@ class JingleTest2:
         # Force Gabble to process the caps before doing any more Jingling
         sync_stream(self.q, self.stream)
 
-    def generate_contents (self, audio = True, video = False):
-        assert audio or video
+    def generate_contents(self):
+        assert len(self.audio_names + self.video_names) > 0
 
         jp = self.jp
 
-        assert not video or jp.can_do_video()
+        assert len(self.video_names) == 0 or jp.can_do_video()
 
         contents = []
 
-        if jp.seperate_contents() or not audio or not video:
-            assert audio or video
-            if audio:
-                contents.append(
-                    jp.Content('stream1', 'initiator', 'both', [
-                        jp.Description('audio', [
-                            jp.PayloadType(name, str(rate), str(id)) for
-                            (name, id, rate) in self.audio_codecs ]),
-                    jp.TransportGoogleP2P() ])
-                )
-            if video:
-                contents.append(
-                    jp.Content('stream2', 'initiator', 'both', [
-                        jp.Description('video', [
-                            jp.PayloadType(name, str(rate), str(id)) for
-                            (name, id, rate) in self.video_codecs ]),
-                    jp.TransportGoogleP2P() ])
-                )
-        else:
+        if not jp.seperate_contents() and self.video_names:
             assert jp.can_do_video()
+            assert self.audio_names
+
             contents.append(
                 jp.Content('stream0', 'initiator', 'both', [
                     jp.Description('video', [
@@ -463,12 +541,31 @@ class JingleTest2:
                         ),
                     jp.TransportGoogleP2P() ])
              )
+        else:
+            def mk_content(name, media, codecs):
+                contents.append(
+                    jp.Content(name, 'initiator', 'both', [
+                        jp.Description(media, [
+                            jp.PayloadType(name, str(rate), str(id)) for
+                            (name, id, rate) in codecs ]),
+                    jp.TransportGoogleP2P() ])
+                )
+
+            for name in self.audio_names:
+                mk_content(name, 'audio', self.audio_codecs)
+
+            for name in self.video_names:
+                mk_content(name, 'video', self.video_codecs)
 
         return contents
 
     def incoming_call(self, audio=True, video=False):
         jp = self.jp
-        contents = self.generate_contents(audio, video)
+
+        self.audio_names = ['audio1'] if audio else []
+        self.video_names = ['video1'] if video else []
+
+        contents = self.generate_contents()
 
         node = jp.SetIq(self.peer, self.jid, [
             jp.Jingle(self.sid, self.peer, 'session-initiate', contents),
@@ -476,14 +573,14 @@ class JingleTest2:
         self.stream.send(jp.xml(node))
 
     def parse_session_initiate (self, query):
-        # Validate the session initiate and get some useful ifo from it
-        self.jp.validate_session_initiate (query)
-        self.sid = self.jp.extract_session_id(query)
+        # Validate the session initiate and get some useful info from it
+        self.sid, self.audio_names, self.video_names = \
+            self.jp.validate_session_initiate(query)
 
-    def accept(self, with_video=False):
+    def accept(self):
         jp = self.jp
 
-        contents = self.generate_contents(True, with_video)
+        contents = self.generate_contents()
         node = jp.SetIq(self.peer, self.jid, [
             jp.Jingle(self.sid, self.peer, 'session-accept',
                 contents) ])
@@ -515,11 +612,15 @@ class JingleTest2:
                 jp.TransportGoogleP2P() ]) ]) ])
         self.stream.send(jp.xml(node))
 
-    def terminate(self, reason=None):
+    def terminate(self, reason=None, text=""):
         jp = self.jp
 
         if reason is not None and jp.is_modern_jingle():
-            body = [("reason", None, {}, [(reason, None, {}, [])])]
+            body = [("reason", None, {},
+                        [(reason, None, {}, []),
+                         ("text", None, {}, [text]),
+                        ]
+                    )]
         else:
             body = []
 
@@ -552,6 +653,7 @@ class JingleTest2:
                 in enumerate(self.remote_transports) ],
             signature='(usuussduss)')
 
+
 def test_dialects(f, dialects):
     for dialect in dialects:
         exec_test(
