@@ -2,17 +2,14 @@
 Regression test for https://bugs.freedesktop.org/show_bug.cgi?id=18918
 """
 
-from gabbletest import exec_test, make_result_iq, sync_stream, \
-        send_error_reply
-from servicetest import make_channel_proxy, unwrap, tp_path_prefix, \
-        call_async, EventPattern
-from twisted.words.xish import domish
-import jingletest
-import gabbletest
 import dbus
-import time
 
-MEDIA_STREAM_TYPE_AUDIO = 0
+from gabbletest import exec_test, sync_stream
+from servicetest import make_channel_proxy
+import jingletest
+import constants as cs
+
+from twisted.words.xish import domish
 
 def test(q, bus, conn, stream):
     jt = jingletest.JingleTest(stream, 'test@localhost', 'foo@bar.com/Foo')
@@ -20,12 +17,8 @@ def test(q, bus, conn, stream):
     # Connecting
     conn.Connect()
 
-    q.expect('dbus-signal', signal='StatusChanged', args=[1, 1])
-
-    q.expect('stream-authenticated')
-    q.expect('dbus-signal', signal='PresenceUpdate',
-        args=[{1L: (0L, {u'available': {}})}])
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     self_handle = conn.GetSelfHandle()
 
@@ -41,11 +34,9 @@ def test(q, bus, conn, stream):
     # Force Gabble to process the caps before calling RequestChannel
     sync_stream(q, stream)
 
-    handle = conn.RequestHandles(1, [jt.remote_jid])[0]
-
+    handle = conn.RequestHandles(cs.HT_CONTACT, [jt.remote_jid])[0]
     path = conn.RequestChannel(
-        'org.freedesktop.Telepathy.Channel.Type.StreamedMedia',
-        1, handle, True)
+        cs.CHANNEL_TYPE_STREAMED_MEDIA, cs.HT_CONTACT, handle, True)
 
     channel = bus.get_object(conn.bus_name, path)
     signalling_iface = make_channel_proxy(conn, path, 'Channel.Interface.MediaSignalling')
@@ -56,7 +47,7 @@ def test(q, bus, conn, stream):
     # send Gabble some XML that will cause an event when processed, and
     # wait for that event (until
     # https://bugs.freedesktop.org/show_bug.cgi?id=15769 is fixed)
-    el = domish.Element(('jabber.client', 'presence'))
+    el = domish.Element(('jabber:client', 'presence'))
     el['from'] = 'bob@example.com/Bar'
     stream.send(el.toXml())
     q.expect('dbus-signal', signal='PresenceUpdate')
@@ -66,7 +57,7 @@ def test(q, bus, conn, stream):
     # Test that codec parameters are correctly sent in <parameter> children of
     # <payload-type> rather than as attributes of the latter.
 
-    media_iface.RequestStreams(handle, [MEDIA_STREAM_TYPE_AUDIO])
+    media_iface.RequestStreams(handle, [cs.MEDIA_STREAM_TYPE_AUDIO])
 
     # S-E gets notified about new session handler, and calls Ready on it
     e = q.expect('dbus-signal', signal='NewSessionHandler')
@@ -84,7 +75,7 @@ def test(q, bus, conn, stream):
     codecs = dbus.Array( [ (96, 'speex', 0, 16000, 0, {'vbr': 'on'}) ],
                          signature='(usuuua{ss})')
     stream_handler.Ready(codecs)
-    stream_handler.StreamState(2)
+    stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
 
     e = q.expect('stream-iq')
     content = list(e.query.elements())[0]
@@ -136,14 +127,5 @@ def test(q, bus, conn, stream):
         assert len(parameters) == 1, parameters
         assert parameters['misc'] == 'other', parameters
 
-    # Test completed, close the connection
-
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
-    return True
-
-
 if __name__ == '__main__':
     exec_test(test)
-
