@@ -6,7 +6,7 @@ import dbus
 import socket
 
 from gabbletest import exec_test, make_result_iq, sync_stream, GoogleXmlStream
-from servicetest import make_channel_proxy, tp_path_prefix, EventPattern
+from servicetest import make_channel_proxy, EventPattern
 import jingletest
 import constants as cs
 
@@ -79,7 +79,7 @@ def test(q, bus, conn, stream,
     e = q.expect('dbus-signal', signal='NewStreamHandler')
     stream_handler = make_channel_proxy(conn, e.args[0], 'Media.StreamHandler')
 
-    media_chan = make_channel_proxy(conn, tp_path_prefix + e.path, 'Channel.Interface.Group')
+    media_chan = make_channel_proxy(conn, e.path, 'Channel.Interface.Group')
 
     # Exercise channel properties
     channel_props = media_chan.GetAll(
@@ -101,18 +101,18 @@ def test(q, bus, conn, stream,
 
     if expected_stun_server == None:
         # If there is no stun server set then gabble should fallback on the
-        # default fallback stunserver (stun.collabora.co.uk)
+        # default fallback stunserver (stun.telepathy.im)
         # This test assumes that if python can resolve the stun servers
         # address then gabble should be able to resolv it as well
         try:
             expected_stun_server = \
-                socket.gethostbyaddr("stun.collabora.co.uk")[2][0]
+                socket.gethostbyname("stun.telepathy.im")
             expected_stun_port = 3478
         except:
             expected_stun_server = None
 
     if expected_stun_server is None:
-        assert sh_props['STUNServers'] == stun_address, sh_props['STUNServers']
+        assert sh_props['STUNServers'] == [], sh_props['STUNServers']
     else:
         assert sh_props['STUNServers'] == \
             [(expected_stun_server, expected_stun_port)], \
