@@ -55,6 +55,12 @@ def test(jp, q, bus, conn, stream, peer_removes_final_content):
     stream_handler2 = make_channel_proxy(conn, e.args[0], 'Media.StreamHandler')
     stream_handler2.NewNativeCandidate("fake", jt.get_remote_transports_dbus())
 
+    # Before the CM can initiate session, we modify a stream direction. This
+    # should result in a no-op since there's no need to inform the peer of
+    # change.
+    chan.StreamedMedia.RequestStreamDirection(stream_id2,
+        cs.MEDIA_STREAM_DIRECTION_RECEIVE)
+
     # We set both streams as ready, which will trigger the session initiate
     stream_handler.Ready(jt.get_audio_codecs_dbus())
     stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
