@@ -6,8 +6,7 @@ Test MUC support.
 import dbus
 
 from gabbletest import make_result_iq, exec_test, sync_stream, disconnect_conn
-from servicetest import call_async, EventPattern, \
-        tp_name_prefix, tp_path_prefix
+from servicetest import call_async, EventPattern, tp_name_prefix
 import constants as cs
 
 def test(q, bus, conn, stream):
@@ -170,10 +169,8 @@ def test(q, bus, conn, stream):
     assert ensured_path == path2, (ensured_path, path2)
 
     disconnect_conn(q, conn, stream, [
-    EventPattern('dbus-signal', signal='Closed',
-        path=path1[len(tp_path_prefix):]),
-    EventPattern('dbus-signal', signal='Closed',
-        path=path2[len(tp_path_prefix):]),
+    EventPattern('dbus-signal', signal='Closed', path=path1),
+    EventPattern('dbus-signal', signal='Closed', path=path2),
     EventPattern('dbus-signal', signal='ChannelClosed', args=[path1]),
     EventPattern('dbus-signal', signal='ChannelClosed', args=[path2])])
 
