@@ -3,13 +3,9 @@ test OLPC Buddy properties change notifications
 """
 # FIXME: merge this file to other tests ?
 
-import dbus
-
-from servicetest import call_async, EventPattern
-from gabbletest import exec_test, make_result_iq, acknowledge_iq
-
-from twisted.words.xish import domish, xpath
-
+from servicetest import EventPattern
+from gabbletest import exec_test, acknowledge_iq
+import constants as cs
 from util import announce_gadget, send_buddy_changed_properties_msg
 import ns
 
@@ -17,7 +13,8 @@ def test(q, bus, conn, stream):
     conn.Connect()
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
