@@ -5,14 +5,14 @@ test OLPC search activity
 import dbus
 
 from servicetest import call_async, EventPattern
-from gabbletest import exec_test, make_result_iq, acknowledge_iq, sync_stream
+from gabbletest import exec_test, acknowledge_iq, sync_stream
 
-from twisted.words.xish import domish, xpath
-from twisted.words.protocols.jabber.client import IQ
+from twisted.words.xish import xpath
 from util import (announce_gadget, request_random_activity_view,
     answer_error_to_pubsub_request, send_reply_to_activity_view_request,
     parse_properties, properties_to_xml, create_gadget_message, close_view)
 import ns
+import constants as cs
 
 tp_name_prefix = 'org.freedesktop.Telepathy'
 olpc_name_prefix = 'org.laptop.Telepathy'
@@ -35,7 +35,8 @@ def test(q, bus, conn, stream):
     handles = {}
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
@@ -46,11 +47,11 @@ def test(q, bus, conn, stream):
             'org.laptop.Telepathy.ActivityProperties')
     buddy_prop_iface = dbus.Interface(conn, 'org.laptop.Telepathy.BuddyInfo')
     gadget_iface = dbus.Interface(conn, 'org.laptop.Telepathy.Gadget')
-    requests_iface = dbus.Interface(conn, tp_name_prefix + '.Connection.Interface.Requests')
+    requests_iface = dbus.Interface(conn, cs.CONN_IFACE_REQUESTS)
 
     # Gadget was not announced yet
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': 5,
           })
@@ -61,10 +62,9 @@ def test(q, bus, conn, stream):
 
     # check if we can request Activity views
     properties = conn.GetAll(
-        'org.freedesktop.Telepathy.Connection.Interface.Requests',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        cs.CONN_IFACE_REQUESTS, dbus_interface=dbus.PROPERTIES_IFACE)
 
-    assert ({tp_name_prefix + '.Channel.ChannelType':
+    assert ({cs.CHANNEL_TYPE:
             olpc_name_prefix + '.Channel.Type.ActivityView'},
 
             [olpc_name_prefix + '.Channel.Interface.View.MaxSize',
@@ -83,9 +83,7 @@ def test(q, bus, conn, stream):
     view1 = bus.get_object(conn.bus_name, view_path)
 
     # check org.freedesktop.Telepathy.Channel D-Bus properties
-    props = view1.GetAll(
-        'org.freedesktop.Telepathy.Channel',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+    props = view1.GetAll(cs.CHANNEL, dbus_interface=dbus.PROPERTIES_IFACE)
 
     assert props['ChannelType'] == 'org.laptop.Telepathy.Channel.Type.ActivityView'
     assert 'org.laptop.Telepathy.Channel.Interface.View' in props['Interfaces']
@@ -108,7 +106,7 @@ def test(q, bus, conn, stream):
     assert props['Properties'] == {}
     assert props['Participants'] == []
 
-    assert view1.GetChannelType(dbus_interface='org.freedesktop.Telepathy.Channel') ==\
+    assert view1.GetChannelType(dbus_interface=cs.CHANNEL) ==\
             'org.laptop.Telepathy.Channel.Type.ActivityView'
 
     ## Current views ##
@@ -167,7 +165,7 @@ def test(q, bus, conn, stream):
     props = dbus.Dictionary({'color': '#AABBCC,#001122'}, signature='sv')
 
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': 5,
             'org.laptop.Telepathy.Channel.Type.ActivityView.Properties': props,
@@ -214,20 +212,18 @@ def test(q, bus, conn, stream):
             args=[[('activity2', handles['room2'])], []])
 
     act = view2.Get(olpc_name_prefix + '.Channel.Interface.View',
-        'Activities',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        'Activities', dbus_interface=dbus.PROPERTIES_IFACE)
     assert sorted(act) == [('activity2', handles['room2'])]
 
     buddies = view2.Get(olpc_name_prefix + '.Channel.Interface.View',
-        'Buddies',
-        dbus_interface=dbus.PROPERTIES_IFACE)
+        'Buddies', dbus_interface=dbus.PROPERTIES_IFACE)
     assert buddies == []
 
     # activity search by participants (view 3)
     participants = conn.RequestHandles(1, ["alice@localhost", "bob@localhost"])
 
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': 5,
             'org.laptop.Telepathy.Channel.Type.ActivityView.Participants': participants,
@@ -502,19 +498,19 @@ def test(q, bus, conn, stream):
 
     # View request without MaxSize property
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
           })
 
     event = q.expect('dbus-error', method='CreateChannel')
-    assert event.error.get_dbus_name() == 'org.freedesktop.Telepathy.Errors.InvalidArgument'
+    assert event.error.get_dbus_name() == cs.INVALID_ARGUMENT
 
     # test participants and properties search
     props = dbus.Dictionary({'color': '#AABBCC,#001122'}, signature='sv')
     participants = conn.RequestHandles(1, ["alice@localhost", "bob@localhost"])
 
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': 5,
             'org.laptop.Telepathy.Channel.Type.ActivityView.Properties': props,
