@@ -6,8 +6,7 @@ Test connecting to a server.
 import dbus
 from twisted.words.xish import xpath
 
-from servicetest import EventPattern
-from gabbletest import go, exec_test
+from gabbletest import exec_test
 import ns
 
 def test(q, bus, conn, stream):
@@ -15,7 +14,6 @@ def test(q, bus, conn, stream):
     buddy_info_iface.SetProperties({'color': '#ff0000,#0000ff'})
 
     conn.Connect()
-    # q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
 
     # buddy activities
     event = q.expect('stream-iq', iq_type='set', query_name='pubsub')
@@ -51,7 +49,6 @@ def test(q, bus, conn, stream):
 
     iq['type'] = 'result'
     stream.send(iq)
-    conn.Disconnect()
 
 if __name__ == '__main__':
     exec_test(test)
