@@ -28,13 +28,6 @@ def test(q, bus, conn, stream):
 
     call_async(q, conn, 'RequestHandles', cs.HT_ROOM, ['chat@conf.localhost'])
 
-    event = q.expect('stream-iq', to='conf.localhost',
-            query_ns='http://jabber.org/protocol/disco#info')
-    result = make_result_iq(stream, event.stanza)
-    feature = result.firstChildElement().addElement('feature')
-    feature['var'] = 'http://jabber.org/protocol/muc'
-    stream.send(result)
-
     event = q.expect('dbus-return', method='RequestHandles')
     handles = event.value[0]
     room_handle = handles[0]
