@@ -1,5 +1,7 @@
 """Test GetAvailableStreamTubeTypes and GetAvailableTubeTypes"""
 
+import sys
+
 import dbus
 
 from servicetest import call_async, EventPattern, tp_name_prefix,\
@@ -28,13 +30,6 @@ def test(q, bus, conn, stream):
 
     call_async(q, conn, 'RequestHandles', cs.HT_ROOM, ['chat@conf.localhost'])
 
-    event = q.expect('stream-iq', to='conf.localhost',
-            query_ns='http://jabber.org/protocol/disco#info')
-    result = make_result_iq(stream, event.stanza)
-    feature = result.firstChildElement().addElement('feature')
-    feature['var'] = 'http://jabber.org/protocol/muc'
-    stream.send(result)
-
     event = q.expect('dbus-return', method='RequestHandles')
     handles = event.value[0]
 
@@ -82,8 +77,14 @@ def test(q, bus, conn, stream):
     # test GetAvailableStreamTubeTypes (old API)
     stream_tubes_types = tubes_iface_muc.GetAvailableStreamTubeTypes()
     assertLength(3, stream_tubes_types)
-    assertEquals([cs.SOCKET_ACCESS_CONTROL_LOCALHOST, cs.SOCKET_ACCESS_CONTROL_CREDENTIALS],
-        stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_UNIX])
+    assert cs.SOCKET_ACCESS_CONTROL_LOCALHOST in \
+        stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_UNIX], \
+        stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_UNIX]
+    # so far we only guarantee to support credentials-passing on Linux
+    if sys.platform == 'linux2':
+        assert cs.SOCKET_ACCESS_CONTROL_CREDENTIALS in \
+            stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_UNIX], \
+            stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_UNIX]
     assertEquals([cs.SOCKET_ACCESS_CONTROL_LOCALHOST, cs.SOCKET_ACCESS_CONTROL_PORT],
         stream_tubes_types[cs.SOCKET_ADDRESS_TYPE_IPV4])
     assertEquals([cs.SOCKET_ACCESS_CONTROL_LOCALHOST, cs.SOCKET_ACCESS_CONTROL_PORT],
